﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using PokemonBattleOnline.Game;

namespace PokemonBattleOnline.Game.Host
{
  internal class Pokemon
  {
    public readonly Controller Controller;
    public readonly int Id;
    public readonly Player Owner;
    public readonly int TeamId;

    public readonly int MaxHp;
    public readonly int AbilityIndex;
    public readonly I6D Iv;
    public readonly I6D Ev;

    public readonly string Name;
    public readonly PokemonGender Gender;
    public readonly int Lv;
    public readonly Move[] Moves;
    public readonly int Happiness;
    public readonly PokemonNature Nature;
    
    public bool Shiny;

    internal Pokemon(Controller controller, int id, Player owner, IPokemonData custom)
    {
      Controller = controller;
      Id = id;
      Owner = owner;
      TeamId = owner.TeamId;

      Form = custom.Form;
      Name = custom.Name;
      Happiness = custom.Happiness;
      Gender = custom.Gender;
      Lv = custom.Lv;
      Nature = custom.Nature;
      AbilityIndex = custom.AbilityIndex;
      Moves = custom.Moves.Select((m) => new Move(m.Move, m.PP)).ToArray();
      _item = custom.Item;
      Iv = new ReadOnly6D(custom.Iv);
      Ev = new ReadOnly6D(custom.Ev);
      _hp = MaxHp = PokemonStatHelper.GetHp(custom.Form.Data.Base.Hp, (byte)Iv.Hp, (byte)Ev.Hp, (byte)Lv);
    }

    public PokemonForm Form; //shaymi
    private int _item;
    public int Item
    {
      get { return _item; }
      set
      {
        if (_item != value)
        {
          _item = value;
          Controller.ReportBuilder.SetItem(this);
        }
      }
    }
    private PokemonState _state;
    public PokemonState State
    {
      get { return _state; }
      set
      {
        if (_state != value)
        {
          _state = value;
          if (value != PokemonState.Faint) Controller.ReportBuilder.SetState(this);
        }
      }
    }

    private bool _mega;
    public bool Mega
    {
      get { return _mega; }
      set
      {
        _mega = value;
        Owner.Mega = true;
      }
    }

    public int IndexInOwner
    { get { return Owner.GetPokemonIndex(Id); } }
    /// <summary>
    /// for PokemonProxy only
    /// </summary>
    public int _hp;
    public int Hp
    { 
      get { return _hp; }
      set
      {
        if (value < 0) value = 0;
        else if (value > MaxHp) value = MaxHp;
        if (_hp != value)
        {
          _hp = value;
          Controller.ReportBuilder.SetHp(this);
        }
      }
    }

    private int Get5D(StatType type)
    {
      return PokemonStatHelper.Get5D(type, Nature, Form.Data.Base.GetStat(type), Iv.GetStat(type), Ev.GetStat(type), Lv);
    }

    /// <summary>
    /// no battle report
    /// </summary>
    public void SetHp(int value)
    {
      if (value < 0) value = 0;
      else if (value > MaxHp) value = MaxHp;
      _hp = value;
    }
  }
}
