﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.ComponentModel;

namespace PokemonBattleOnline.Game
{
  public interface IBoardOutwardEvents
  {
    void PokemonSentout(int team, int x);
  }

  public class BoardOutward
  {
    public readonly ReadOnlyObservableCollection<PokemonOutward>[] Pokemons;
    public readonly Terrain Terrain;

    private readonly ObservableCollection<PokemonOutward>[] pokemons;
    private readonly IGameSettings Settings;
    
    private IBoardOutwardEvents listener;

    internal BoardOutward(IGameSettings settings)
    {
      Settings = settings;
      Teams = new TeamOutward[Settings.Mode.TeamCount()];
      pokemons = new ObservableCollection<PokemonOutward>[settings.Mode.TeamCount()];
      Pokemons = new ReadOnlyObservableCollection<PokemonOutward>[settings.Mode.TeamCount()];
      _weather = Weather.Normal;
      Terrain = settings.Terrain;

      var empty = new PokemonOutward[settings.Mode.XBound()];
      for (int i = 0; i < settings.Mode.TeamCount(); i++)
      {
        pokemons[i] = new ObservableCollection<PokemonOutward>(empty);
        Pokemons[i] = new ReadOnlyObservableCollection<PokemonOutward>(pokemons[i]);
      }
    }

    public PokemonOutward this[int team, int x]
    {
      get { return pokemons[team][x]; }
      set
      {
        //不一定是PmSendOut
        var old = this[team, x];
        if ((old == null && value == null) || ((old != null && value != null) && (old.Id == value.Id))) return;
        pokemons[team][x] = value;
      }
    }
    private Weather _weather;
    public Weather Weather
    {
      get { return _weather; }
      internal set
      {
        _weather = value;
      }
    }
    public TeamOutward[] Teams
    { get; private set; }

    public void AddListener(IBoardOutwardEvents listener)
    {
      if (this.listener != null) System.Diagnostics.Debugger.Break();
      this.listener = listener;
    }
    public void PokemonSentout(GameOutward game, int team, int x)
    {
      listener.PokemonSentout(team, x);
    }
  }
}
